/*
 * Copyright (C) 2012 OUYA, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package tv.ouya.sample;

import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import com.google.common.collect.Lists;
import com.xtremelabs.robolectric.RobolectricTestRunner;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.ArrayList;
import java.util.Arrays;

import static com.xtremelabs.robolectric.Robolectric.shadowOf;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertSame;
import static junit.framework.Assert.assertTrue;

@RunWith(RobolectricTestRunner.class)
public class GenericAdapterTest {
    private ArrayList<String> strings;
    private TestAdapter adapter;

    @Before
    public void setUp() throws Exception {
        strings = new ArrayList<String>();
        strings.add("one");
        strings.add("two");
        strings.add("three");
        adapter = new TestAdapter();
        adapter.add(strings);
    }

    @Test
    public void getCount_shouldReturnListSize() throws Exception {
        assertEquals(3, adapter.getCount());
        shadowOf(adapter).clearWasDataSetChangedCalledFlag();
        adapter.add(Arrays.asList("four"));
        assertEquals(4, adapter.getCount());
        assertTrue(shadowOf(adapter).wasNotifyDataSetChangedCalled());
    }

    @Test
    public void getItem_shouldReturnObjectAtIndex() throws Exception {
        assertEquals("one", adapter.getItem(0));
        assertEquals("two", adapter.getItem(1));
        assertEquals("three", adapter.getItem(2));
    }

    @Test
    public void getItemId_shouldReturnIndex() throws Exception {
        assertEquals(666L, adapter.getItemId(666));
    }

    @Test
    public void getView_shouldCallThroughToGetView() throws Exception {
        View convertView = new View(null);
        ViewGroup parent = new LinearLayout(null);

        adapter.getView(1, convertView, parent);

        assertEquals("two", adapter.item);
        assertSame(convertView, adapter.recycleView);
        assertSame(parent, adapter.parent);
    }

    @Test
    public void set_shouldDisplayTheNewContents() throws Exception {
        adapter.set(Lists.newArrayList("only"));
        assertEquals(1, adapter.getCount());
        assertEquals("only", adapter.getItem(0));
        assertTrue(shadowOf(adapter).wasNotifyDataSetChangedCalled());
    }

    @Test
    public void add_notifiesDataSetChanged() throws Exception {
        shadowOf(adapter).clearWasDataSetChangedCalledFlag();
        adapter.add(Lists.newArrayList("Four"));
        assertTrue(shadowOf(adapter).wasNotifyDataSetChangedCalled());
    }

    static class TestAdapter extends GenericAdapter<String> {
        private String item;
        private View recycleView;
        private ViewGroup parent;

        @Override
        public View getView(String item, View recycleView, ViewGroup parent) {
            this.item = item;
            this.recycleView = recycleView;
            this.parent = parent;
            return null;
        }
    }
}
